import unittest

import pandas as pd
import pytest
from pandas.testing import assert_frame_equal

from hydrotoolbox import indices
from hydrotoolbox.toolbox_utils.src.toolbox_utils import tsutils

all = [
    "MA1",
    "MA2",
    "MA3",
    "MA4",
    "MA5",
    "MA6",
    "MA7",
    "MA8",
    "MA9",
    "MA10",
    "MA11",
    "MA12",
    "MA13",
    "MA14",
    "MA15",
    "MA16",
    "MA17",
    "MA18",
    "MA19",
    "MA20",
    "MA21",
    "MA22",
    "MA23",
    "MA24",
    "MA25",
    "MA26",
    "MA27",
    "MA28",
    "MA29",
    "MA30",
    "MA31",
    "MA32",
    "MA33",
    "MA34",
    "MA35",
    "MA36",
    "MA37",
    "MA38",
    "MA39",
    "MA40",
    "MA41",
    "MA42",
    "MA43",
    "MA44",
    "MA45",
    "ML1",
    "ML2",
    "ML3",
    "ML4",
    "ML5",
    "ML6",
    "ML7",
    "ML8",
    "ML9",
    "ML10",
    "ML11",
    "ML12",
    "ML13",
    "ML14",
    "ML15",
    "ML16",
    "ML17",
    "ML18",
    "ML19",
    "ML20",
    "ML21",
    "ML22",
    "MH1",
    "MH2",
    "MH3",
    "MH4",
    "MH5",
    "MH6",
    "MH7",
    "MH8",
    "MH9",
    "MH10",
    "MH11",
    "MH12",
    "MH13",
    "MH14",
    "MH15",
    "MH16",
    "MH17",
    "MH18",
    "MH19",
    "MH20",
    "MH21",
    "MH22",
    "MH23",
    "MH24",
    "MH25",
    "MH26",
    "MH27",
    "FL1",
    "FL2",
    "FL3",
    "FH1",
    "FH2",
    "FH3",
    "FH4",
    "FH5",
    "FH6",
    "FH7",
    "FH8",
    "FH9",
    "FH10",
    "DL1",
    "DL2",
    "DL3",
    "DL4",
    "DL5",
    "DL6",
    "DL7",
    "DL8",
    "DL9",
    "DL10",
    "DL11",
    "DL12",
    "DL13",
    "DL14",
    "DL15",
    "DL16",
    "DL17",
    "DL18",
    "DL19",
    "DL20",
    "DH1",
    "DH2",
    "DH3",
    "DH4",
    "DH5",
    "DH6",
    "DH7",
    "DH8",
    "DH9",
    "DH10",
    "DH11",
    "DH12",
    "DH13",
    "DH14",
    "DH15",
    "DH16",
    "DH17",
    "DH18",
    "DH19",
    "DH20",
    "DH21",
    "TA1",
    "TA2",
    "TL1",
    "TL2",
    "TH1",
    "TH2",
    "RA1",
    "RA2",
    "RA3",
    "RA4",
    "RA5",
    "RA6",
    "RA7",
    "RA8",
    "RA9",
]


# Helper function to create a natural sorting key
def natural_keys(text):
    def atoi(text):
        return int(text) if text.isdigit() else text

    return [atoi(c) for c in re.split(r"(\d+)", text)]


_all_large_reference = {
    "MA1: Mean of all daily flows": 36.07385680338878,
    "MA2: Median of all daily flows": 31.0,
    "MA3: CV of all daily flows": 64.38728590511374,
    "MA4: CV of  log of all daily flows": 35.92322864951162,
    "MA5: Mean daily flow/median daily flow": 1.1636728001093155,
    "MA6: Q10/Q90 for all daily flows": 2.6,
    "MA7: Q20/Q80 for all daily flows": 1.8695652173913044,
    "MA8: Q25/Q75 for all daily flows": 1.6666666666666667,
    "MA9: (Q10-Q90)/median daily flow": 1.032258064516129,
    "MA10: (Q20-Q80)/median daily flow": 0.6451612903225806,
    "MA11: (Q25-Q75)/median daily flow": 0.5161290322580645,
    "MA12: Mean monthly flow: January": 29.38990870359099,
    "MA13: Mean monthly flow: February": 34.074822979291916,
    "MA14: Mean monthly flow: March": 47.93560559951308,
    "MA15: Mean monthly flow: April": 42.80816352201258,
    "MA16: Mean monthly flow: May": 40.01236153377967,
    "MA17: Mean monthly flow: June": 38.927754716981134,
    "MA18: Mean monthly flow: July": 36.24406573341449,
    "MA19: Mean monthly flow: August": 34.477784540474744,
    "MA20: Mean monthly flow: September": 33.16792452830189,
    "MA21: Mean monthly flow: October": 31.630030432136337,
    "MA22: Mean monthly flow: November": 33.41174213836478,
    "MA23: Mean monthly flow: December": 30.75816189896531,
    "MA24: CV of monthly flow: January": 21.255576370855696,
    "MA25: CV of monthly flow: February": 44.384550841685446,
    "MA26: CV of monthly flow: March": 54.340282672861036,
    "MA27: CV of monthly flow: April": 28.470253027401405,
    "MA28: CV of monthly flow: May": 25.779474313908885,
    "MA29: CV of monthly flow: June": 32.630383087795046,
    "MA30: CV of monthly flow: July": 25.286964852579075,
    "MA31: CV of monthly flow: August": 26.57473040113124,
    "MA32: CV of monthly flow: September": 21.05548162143981,
    "MA33: CV of monthly flow: October": 16.242845646635086,
    "MA34: CV of monthly flow: November": 18.170764243071073,
    "MA35: CV of monthly flow: December": 15.502441372622572,
    "MA36: Range mean monthly/median monthly flow": 3.825979233565339,
    "MA37: IQR mean monthly/median monthly flow": 0.531280053464604,
    "MA38: (Q10-Q90)[monthly]/median monthly flow": 0.9844007237281781,
    "MA39: CV: monthly mean flows": 41.69146860072855,
    "MA40: Skewness in monthly flows": 0.09358720722623534,
    "MA41: Mean annual runoff": 36.178807097722455,
    "MA42: Range mean annual/median annual flow": 1.1347929675493067,
    "MA43: IQR mean annual/median annual flow": 0.41261609375776315,
    "MA44: (Q10-Q90)[annual]/median annual flow": 0.6630643371397152,
    "MA45: Skewness in annual flows": -0.0036254793502020857,
    "ML1: Mean minimum monthly flow: January": 24.997358490566036,
    "ML2: Mean minimum monthly flow: February": 24.674716981132075,
    "ML3: Mean minimum monthly flow: March": 28.098301886792452,
    "ML4: Mean minimum monthly flow: April": 31.782830188679245,
    "ML5: Mean minimum monthly flow: May": 30.274150943396226,
    "ML6: Mean minimum monthly flow: June": 28.50943396226415,
    "ML7: Mean minimum monthly flow: July": 27.433962264150942,
    "ML8: Mean minimum monthly flow: August": 26.62264150943396,
    "ML9: Mean minimum monthly flow: September": 26.41509433962264,
    "ML10: Mean minimum monthly flow: October": 27.024150943396226,
    "ML11: Mean minimum monthly flow: November": 27.700377358490563,
    "ML12: Mean minimum monthly flow: December": 26.53679245283019,
    "ML13: CV of minimum monthly flows": 31.023453880562577,
    "ML14: Mean minimum daily flow/mean median annual flow": 0.6944306960202719,
    "ML15: Mean minimum annual flow/mean annual flow": 0.6007523328104921,
    "ML16: Median minimum annual flow/median annual flow": 0.7267400568181819,
    "ML17: 7-day minimum flow/mean annual flow": 0.6243949555991795,
    "ML18: CV of (7-day minimum flow/mean annual)": 15.488217919532971,
    "ML19: Mean of (minimum annual flow/mean annual)*100": 60.075233281049215,
    "ML20: Ratio of baseflow volume to total flow volume": 0.824170109368937,
    "ML21: CV of annual minimum flows": 27.62247777349517,
    "ML22: Mean annual minimum flow divided by catchment area": 21.4712962962963,
    "MH1: Mean maximum monthly flow: January": 59.57811320754717,
    "MH2: Mean maximum monthly flow: February": 103.32264150943396,
    "MH3: Mean maximum monthly flow: March": 164.0498113207547,
    "MH4: Mean maximum monthly flow: April": 97.24018867924528,
    "MH5: Mean maximum monthly flow: May": 90.85773584905661,
    "MH6: Mean maximum monthly flow: June": 106.56603773584905,
    "MH7: Mean maximum monthly flow: July": 93.09433962264151,
    "MH8: Mean maximum monthly flow: August": 92.9622641509434,
    "MH9: Mean maximum monthly flow: September": 63.886792452830186,
    "MH10: Mean maximum monthly flow: October": 54.76679245283019,
    "MH11: Mean maximum monthly flow: November": 58.92396226415094,
    "MH12: Mean maximum monthly flow: December": 47.90735849056604,
    "MH13: CV of maximum monthly flows": 124.35575549955398,
    "MH14: Median maximum annual flow/median annual flow": 9.014354066985646,
    "MH15: Mean of Q1 values/median daily flow across all years": 4.193548387096774,
    "MH16: Mean of Q10 values/median daily flow across all years": 1.6774193548387097,
    "MH17: Mean of Q25 values/median daily flow across all years": 1.2903225806451613,
    "MH18: CV of logarithmic annual maximum flows": 11.860312544136873,
    "MH19: Skewness in annual maximum flows": -0.26782989731354506,
    "MH20: Mean annual maximum flow/catchment area": 320.4259259259259,
    "MH21: High-flow volume (thresh=1*median annual)": 9.776768943592137,
    "MH22: High-flow volume (thresh=3*median annual)": 4.753048309924371,
    "MH23: High-flow volume (thresh=7*median annual)": 6.279156327543426,
    "MH24: Maximum peak flow/median flow (thresh=1*median annual)": 2.0842141935483873,
    "MH25: Maximum peak flow/median flow (thresh=3*median annual)": 6.245454545454545,
    "MH26: Maximum peak flow/median flow (thresh=7*median annual)": 12.144563918757466,
    "MH27: Maximum peak flow/median flow (threshold=Q25)": 2.6287788845775175,
    "FL1: Annual low flow pulse count; number of periods<25th percentile": 6.388888888888889,
    "FL2: CV of low flow pulse count": 105.958447696724,
    "FL3: Count of low flow spells (<5% of mean)/record length (yrs)": 0.0,
    "FH1: Annual high flow pulse count; number of periods>75th percentile": 10.685185185185185,
    "FH2: CV of high flow pulse count": 46.981409828023786,
    "FH3: Count of high flow events (>3*median annual)": 7.185185185185185,
    "FH4: Count of high flow events (>7*median annual)": 1.2962962962962963,
    "FH5: Count of high flow events (>1*median annual)/record length (yrs)": 10.592592592592593,
    "FH6: Count of high flow events (>3*median annual)/record length (yrs)": 3.8703703703703702,
    "FH7: Count of high flow events (>7*median annual)/record length (yrs)": 0.9629629629629629,
    "FH8: Count of high flow events (>25th percentile)/record length (yrs)": 10.685185185185185,
    "FH9: Count of high flow events (>75th percentile)/record length (yrs)": 7.333333333333333,
    "FH10: Count of high flow events (>median of annual minima)/record length (yrs)": 5.407407407407407,
    "DL1: Annual minimum of 1-day mean of flow": 21.4712962962963,
    "DL2: Annual minimum of 3-day mean of flow": 21.829629629629633,
    "DL3: Annual minimum of 7-day mean of flow": 22.33759259259259,
    "DL4: Annual minimum of 30-day mean of flow": 24.0128950617284,
    "DL5: Annual minimum of 90-day mean of flow": 27.291862139917693,
    "DL6: CV: annual minimum of 1-day mean of flow": 27.622477773495174,
    "DL7: CV: annual minimum of 3-day mean of flow": 27.189191888071623,
    "DL8: CV: annual minimum of 7-day mean of flow": 26.50581934541782,
    "DL9: CV: annual minimum of 30-day mean of flow": 26.95798351626695,
    "DL10: CV: annual minimum of 90-day mean of flow": 27.2961216054927,
    "DL11: Mean of 1-day minimum of flow": 0.6926224611708484,
    "DL12: Mean of 7-day minimum of flow": 0.7205675029868577,
    "DL13: Mean of 30-day minimum of flow": 0.774609518120271,
    "DL14: Mean of Q75 values/median daily flow across all years": 0.7741935483870968,
    "DL15: Mean of Q90 values/median daily flow across all years": 0.6451612903225806,
    "DL16: Low flow pulse duration (Mean duration of FL1)": 9.909090909090908,
    "DL17: CV: low flow pulse duration (DL16)": 76.52035207059103,
    "DL18: Mean annual number of zero-flow days": 0,
    "DL19: CV: mean annual number of zero-flow days": 0,
    "DL20: Percentage of all months with zero flow": 0,
    "DH1: Annual maximum of 1-day mean of flow": 320.4259259259259,
    "DH2: Annual maximum of 3-day mean of flow": 190.62962962962962,
    "DH3: Annual maximum of 7-day mean of flow": 124.42253968253969,
    "DH4: Annual maximum of 30-day mean of flow": 68.76645061728394,
    "DH5: Annual maximum of 90-day mean of flow": 49.518378600823056,
    "DH6: CV: annual maximum of 1-day mean of flow": 66.48936587576621,
    "DH7: CV: annual maximum of 3-day mean of flow": 58.41005168523512,
    "DH8: CV: annual maximum of 7-day mean of flow": 51.699295971084005,
    "DH9: CV: annual maximum of 30-day mean of flow": 37.46746622517368,
    "DH10: CV: annual maximum of 90-day mean of flow": 29.437651098757726,
    "DH11: Mean of 1-day maximum of flow": 10.3363201911589,
    "DH12: Mean of 7-day maximum of flow": 4.01363031233999,
    "DH13: Mean of 30-day maximum of flow": 2.218272600557546,
    "DH14: Q95 value/mean monthly flow across all years": 1.7583343751401086,
    "DH15: Mean duration of flood pulses>75th percentile flow": 5.477272727272727,
    "DH16: CV: mean duration of high flow pulse (FH1)": 94.67992784049474,
    "DH17: Mean duration of flood pulses>1*median flow": 39.2325717443859,
    "DH18: Mean duration of flood pulses>3*median flow": 1.7628611040375746,
    "DH19: Mean duration of flood pulses>7*median flow": 1.349462365591398,
    "DH20: Mean duration of flood pulses>25th percentile of median flow": 8.260814157415036,
    "DH21: Mean duration of flood pulses>75th percentile of median flow": 124.43576135090369,
    "TA1: Constancy (see Colwell: 1974)": 0.6799490460176979,
    "TA2: Predictability of flow": 70.54309513427759,
    "TL1: Mean day-of-year of annual minimum": 270.7701089086908,
    "TL2: CV: day-of-year of annual minimum": 70.73937406124507,
    "TH1: Mean day-of-year of annual maximum": 101.51600963786183,
    "TH2: CV: day-of-year of annual maximum": 62.671954877334244,
    "RA1: Mean of positive changes from one day to next (rise rate)": 11.540938073394495,
    "RA2: CV: mean of positive changes from one day to next (rise rate)": 341.119991227145,
    "RA3: Mean of negative changes from one day to next (fall rate)": 6.101332929047907,
    "RA4: CV: mean of negative changes from one day to next (fall rate)": 403.1979503132954,
    "RA5: Ratio of days that are higher than previous day": 0.22522987911974376,
    "RA6: Median of difference in log of flows over two consecutive days of rising flow": 0.06453852113757108,
    "RA7: Median of difference in log of flows over two consecutive days of falling flow": 0.04879016416943216,
    "RA8: Number of flow reversals from one day to the next": 95.51851851851852,
    "RA9: CV: number of flow reversals from one day to the next": 19.31035208429119,
}

_tsproc_reference = {
    "MA1: Mean of all daily flows": 36.03360,
    "MA2: Median of all daily flows": 31.00000,
    "MA3: CV of all daily flows": 65.98392,
    "MA4: CV of  log of all daily flows": 35.86782,
    "MA5: Mean daily flow/median daily flow": 1.162374,
    "MA6: Q10/Q90 for all daily flows": 2.600000,
    "MA7: Q20/Q80 for all daily flows": 1.869565,
    "MA8: Q25/Q75 for all daily flows": 1.666667,
    "MA9: (Q10-Q90)/median daily flow": 1.032258,
    "MA10: (Q20-Q80)/median daily flow": 0.6451613,
    "MA11: (Q25-Q75)/median daily flow": 0.5161290,
    "MA12: Mean monthly flow: January": 28.32118,
    "MA13: Mean monthly flow: February": 33.28521,
    "MA14: Mean monthly flow: March": 46.60481,
    "MA15: Mean monthly flow: April": 41.75635,
    "MA16: Mean monthly flow: May": 39.05942,
    "MA17: Mean monthly flow: June": 38.36068,
    "MA18: Mean monthly flow: July": 36.09560,
    "MA19: Mean monthly flow: August": 33.73490,
    "MA20: Mean monthly flow: September": 32.49515,
    "MA21: Mean monthly flow: October": 31.28219,
    "MA22: Mean monthly flow: November": 32.70525,
    "MA23: Mean monthly flow: December": 30.12355,
    "MA24: CV of monthly flow: January": 20.48265,
    "MA25: CV of monthly flow: February": 43.05685,
    "MA26: CV of monthly flow: March": 52.51972,
    "MA27: CV of monthly flow: April": 27.73464,
    "MA28: CV of monthly flow: May": 25.63139,
    "MA29: CV of monthly flow: June": 33.16160,
    "MA30: CV of monthly flow: July": 27.62596,
    "MA31: CV of monthly flow: August": 25.71185,
    "MA32: CV of monthly flow: September": 21.22240,
    "MA33: CV of monthly flow: October": 17.19186,
    "MA34: CV of monthly flow: November": 17.59787,
    "MA35: CV of monthly flow: December": 15.01697,
    "MA36: Range mean monthly/median monthly flow": 3.843458,
    "MA37: IQR mean monthly/median monthly flow": 0.5330932,
    "MA38: (Q10-Q90)[monthly]/median monthly flow": 0.9859374,
    "MA39: CV: monthly mean flows": 41.62593,
    "MA40: Skewness in monthly flows": 0.9730989e-01,
    "MA41: Mean annual runoff": 0.3614435,
    "MA42: Range mean annual/median annual flow": 1.136707,
    "MA43: IQR mean annual/median annual flow": 0.4083103,
    "MA44: (Q10-Q90)[annual]/median annual flow": 0.6616582,
    "MA45: Skewness in annual flows": -0.2895648e-02,
    "ML1: Mean minimum monthly flow: January": 24.99736,
    "ML2: Mean minimum monthly flow: February": 24.62519,
    "ML3: Mean minimum monthly flow: March": 27.92981,
    "ML4: Mean minimum monthly flow: April": 31.62019,
    "ML5: Mean minimum monthly flow: May": 30.10241,
    "ML6: Mean minimum monthly flow: June": 28.46296,
    "ML7: Mean minimum monthly flow: July": 27.48148,
    "ML8: Mean minimum monthly flow: August": 26.59259,
    "ML9: Mean minimum monthly flow: September": 26.37037,
    "ML10: Mean minimum monthly flow: October": 27.04222,
    "ML11: Mean minimum monthly flow: November": 27.68741,
    "ML12: Mean minimum monthly flow: December": 26.50833,
    "ML13: CV of minimum monthly flows": 30.88031,
    "ML14: Mean minimum daily flow/mean median annual flow": 0.6945993,
    "ML15: Mean minimum annual flow/mean annual flow": 0.5998878,
    "ML16: Median minimum annual flow/median annual flow": 0.7262074,
    "ML17: 7-day minimum flow/mean annual flow": 0.6252760,
    "ML18: CV of (7-day minimum flow/mean annual)": 15.33697,
    "ML19: Mean of (minimum annual flow/mean annual)*100": 59.98878,
    "ML20: Ratio of baseflow volume to total flow volume": 0.8229821,
    "ML21: CV of annual minimum flows": 27.46697,
    "ML22: Mean annual minimum flow divided by catchment area": 0.2142636,
    "MH1: Mean maximum monthly flow: January": 59.57811,
    "MH2: Mean maximum monthly flow: February": 102.1870,
    "MH3: Mean maximum monthly flow: March": 161.5304,
    "MH4: Mean maximum monthly flow: April": 96.23574,
    "MH5: Mean maximum monthly flow: May": 90.73074,
    "MH6: Mean maximum monthly flow: June": 109.4074,
    "MH7: Mean maximum monthly flow: July": 103.6852,
    "MH8: Mean maximum monthly flow: August": 91.87037,
    "MH9: Mean maximum monthly flow: September": 64.70370,
    "MH10: Mean maximum monthly flow: October": 58.03037,
    "MH11: Mean maximum monthly flow: November": 58.42537,
    "MH12: Mean maximum monthly flow: December": 47.55722,
    "MH13: CV of maximum monthly flows": 125.2846,
    "MH14: Median maximum annual flow/median annual flow": 9.210526,
    "MH15: Mean of Q1 values/median daily flow across all years": 4.196452,
    "MH16: Mean of Q10 values/median daily flow across all years": 1.677419,
    "MH17: Mean of Q25 values/median daily flow across all years": 1.290323,
    "MH18: CV of logarithmic annual maximum flows": 11.92860,
    "MH19: Skewness in annual maximum flows": -0.2824223,
    "MH20: Mean annual maximum flow/catchment area": 3.266909,
    "MH21: High-flow volume (thresh=1*median annual)": 9.330008,
    "MH22: High-flow volume (thresh=3*median annual)": 4.771043,
    "MH23: High-flow volume (thresh=7*median annual)": 6.232845,
    "MH24: Maximum peak flow/median flow (thresh=1*median annual)": 2.589813,
    "MH25: Maximum peak flow/median flow (thresh=3*median annual)": 6.289422,
    "MH26: Maximum peak flow/median flow (thresh=7*median annual)": 12.34721,
    "MH27: Maximum peak flow/median flow (threshold=Q25)": 3.031010,
    "FL1: Annual low flow pulse count; number of periods<25th percentile": 6.381818,
    "FL2: CV of low flow pulse count": 105.0923,
    "FL3: Count of low flow spells (<5% of mean)/record length (yrs)": 0.000000,
    "FH1: Annual high flow pulse count; number of periods>75th percentile": 10.69091,
    "FH2: CV of high flow pulse count": 46.41654,
    "FH3: Count of high flow events (>3*median annual)": 7.181818,
    "FH4: Count of high flow events (>7*median annual)": 1.327273,
    "FH5: Count of high flow events (>1*median annual)/record length (yrs)": 10.67273,
    "FH6: Count of high flow events (>3*median annual)/record length (yrs)": 3.909091,
    "FH7: Count of high flow events (>7*median annual)/record length (yrs)": 1.000000,
    "FH8: Count of high flow events (>25th percentile)/record length (yrs)": 10.69091,
    "FH9: Count of high flow events (>75th percentile)/record length (yrs)": 7.345455,
    "FH10: Count of high flow events (>median of annual minima)/record length (yrs)": 5.400000,
    "DL1: Annual minimum of 1-day mean of flow": 21.42636,
    "DL2: Annual minimum of 3-day mean of flow": 21.80242,
    "DL3: Annual minimum of 7-day mean of flow": 22.31241,
    "DL4: Annual minimum of 30-day mean of flow": 23.95308,
    "DL5: Annual minimum of 90-day mean of flow": 27.06629,
    "DL6: CV: annual minimum of 1-day mean of flow": 27.46697,
    "DL7: CV: annual minimum of 3-day mean of flow": 26.96810,
    "DL8: CV: annual minimum of 7-day mean of flow": 26.29971,
    "DL9: CV: annual minimum of 30-day mean of flow": 26.63228,
    "DL10: CV: annual minimum of 90-day mean of flow": 26.58565,
    "DL11: Mean of 1-day minimum of flow": 0.6911730,
    "DL12: Mean of 7-day minimum of flow": 0.7197551,
    "DL13: Mean of 30-day minimum of flow": 0.7726800,
    "DL14: Mean of Q75 values/median daily flow across all years": 0.7741935,
    "DL15: Mean of Q90 values/median daily flow across all years": 0.6451613,
    "DL16: Low flow pulse duration (Mean duration of FL1)": 9.621212,
    "DL17: CV: low flow pulse duration (DL16)": 76.07716,
    "DL18: Mean annual number of zero-flow days": 0.000000,
    "DL19: CV: mean annual number of zero-flow days": 999999.0,
    "DL20: Percentage of all months with zero flow": 1.969697,
    "DH1: Annual maximum of 1-day mean of flow": 326.6909,
    "DH2: Annual maximum of 3-day mean of flow": 194.4515,
    "DH3: Annual maximum of 7-day mean of flow": 125.9205,
    "DH4: Annual maximum of 30-day mean of flow": 68.96815,
    "DH5: Annual maximum of 90-day mean of flow": 49.53489,
    "DH6: CV: annual maximum of 1-day mean of flow": 66.15448,
    "DH7: CV: annual maximum of 3-day mean of flow": 58.14409,
    "DH8: CV: annual maximum of 7-day mean of flow": 50.96947,
    "DH9: CV: annual maximum of 30-day mean of flow": 36.97634,
    "DH10: CV: annual maximum of 90-day mean of flow": 29.15202,
    "DH11: Mean of 1-day maximum of flow": 10.53842,
    "DH12: Mean of 7-day maximum of flow": 4.061951,
    "DH13: Mean of 30-day maximum of flow": 2.224779,
    "DH14: Q95 value/mean monthly flow across all years": 0.000000,
    "DH15: Mean duration of flood pulses>75th percentile flow": 5.454545,
    "DH16: CV: mean duration of high flow pulse (FH1)": 95.40519,
    "DH17: Mean duration of flood pulses>1*median flow": 38.60577,
    "DH18: Mean duration of flood pulses>3*median flow": 1.752998,
    "DH19: Mean duration of flood pulses>7*median flow": 1.338542,
    "DH20: Mean duration of flood pulses>25th percentile of median flow": 8.159773,
    "DH21: Mean duration of flood pulses>75th percentile of median flow": 122.9437,
    "TA1: Constancy (see Colwell: 1974)": 0.6810596,
    "TA2: Predictability of flow": 70.57691,
    "TL1: Mean day-of-year of annual minimum": 273.0798,
    "TL2: CV: day-of-year of annual minimum": 71.51193,
    "TH1: Mean day-of-year of annual maximum": 104.7312,
    "TH2: CV: day-of-year of annual maximum": 62.99718,
    "RA1: Mean of positive changes from one day to next (rise rate)": 11.74332,
    "RA2: CV: mean of positive changes from one day to next (rise rate)": 344.2562,
    "RA3: Mean of negative changes from one day to next (fall rate)": 6.209728,
    "RA4: CV: mean of negative changes from one day to next (fall rate)": 407.8697,
    "RA5: Ratio of days that are higher than previous day": 0.2244566,
    "RA6: Median of difference in log of flows over two consecutive days of rising flow": 0.6466216e-01,
    "RA7: Median of difference in log of flows over two consecutive days of falling flow": 0.4879016e-01,
    "RA8: Number of flow reversals from one day to the next": 94.49091,
    "RA9: CV: number of flow reversals from one day to the next": 19.60414,
}


class TestIndices(unittest.TestCase):
    def test_all_large_dataset(self):
        # Arrange
        # No arrangement needed as input values are provided via test parameters

        # Act
        result = indices(
            all,
            input_ts="Q_BEC_BE_6500.csv",
            start_date="1955-01-01",
        )
        # Assert
        self.assertDictEqual(result, _all_large_reference)

    def test_all_large_dataset_tsproc(self):
        # Arrange
        # No arrangement needed as input values are provided via test parameters

        # Act
        result = indices(
            all,
            input_ts="Q_BEC_BE_6500.csv",
            start_date="1955-01-01",
        )
        # Assert
        missing = [
            2,
            11,
            12,
            13,
            14,
            15,
            16,
            17,
            18,
            19,
            20,
            21,
            22,
            23,
            24,
            25,
            26,
            28,
            29,
            30,
            32,
            33,
            34,
            39,
            40,
            42,
            44,
            66,
            68,
            69,
            70,
            72,
            73,
            74,
            75,
            76,
            80,
            85,
            86,
            87,
            90,
            92,
            93,
            98,
            100,
            103,
            115,
            116,
            122,
            125,
            126,
            127,
            128,
            129,
            134,
            135,
            137,
            138,
            140,
            143,
            146,
            147,
            151,
            152,
            154,
            156,
            157,
            161,
            162,
        ]

        res = pd.Series(result)
        res = res.reset_index()
        ref = pd.Series(_tsproc_reference)
        ref = ref.reset_index()

        for ln in missing:
            res.iloc[ln, :] = ref.iloc[ln, :]

        assert_frame_equal(res, ref, rtol=0.01)
